/******************************************************************************
 *
 * Copyright 2013, Silicon Image, Inc.  All rights reserved.
 * No part of this work may be reproduced, modified, distributed, transmitted,
 * transcribed, or translated into any language or computer format, in any form
 * or by any means without written permission of
 * Silicon Image, Inc., 1140 East Arques Avenue, Sunnyvale, California 94085
 *
 *****************************************************************************/
/**
 * @file Sii977xLib.c
 *
 * @brief Defines the exported functions for the DLL application.
 *
 *****************************************************************************/
#define SII_DEBUG 0

/***** #include statements ***************************************************/

#include <stdio.h>
#include <string.h>

#include "si_datatypes.h"
#include "si_system_api.h"
#include "sii9777rx_api.h"
#include "si_drv_cra_api.h"
#include "si_drv_isp_api.h"
#include "si_drv_ipc_api.h"
#include "sii9777_platform_api.h"
#include "sii9777rx_ipc_def.h"
#include "sii9777_user_def.h"

/***** Register Module name **************************************************/

SII_MODULE_NAME_SET(common_api);

/***** local macro definitions ***********************************************/

#ifndef SII9777_USER_DEF__MULTI_THREAD
#define SII9777_USER_DEF__MULTI_THREAD          0
#endif

#define RESULT_CODE__CMD_SUCCESS                ((uint8_t)0x00)
#define RESULT_CODE__CMD_ERR_FAIL               ((uint8_t)0xFF)

#define EVENT_MASK_INTERNAL                     (SII9777_EVENT_FLAGS__FLASH_DONE)
#define EVENT_MASK_EXTERNAL                     (~EVENT_MASK_INTERNAL)

#define IPC_COMPLETION_TIMEOUT                  1800   /* milliseconds */
#define BOOT_TIME_MAX                           5000   /* milliseconds */

#define RETURN_API(retval)                      { if( retval ) { SII_LOG1B(("(RETVAL=%08lX)\n", retval)); } return retval; }

//#define SII977X_LIB_ES0_MHL
#ifdef SII977X_LIB_ES0_MHL
#include "sii9777mhl_api.h"
#endif
#define CBUS_DEVCAP_LEN                         16
#define CBUS_EXTENDED_DEVCAP_LEN                4

#if( SII9777_USER_DEF__MULTI_THREAD )
#define MUTEX_HANDLE_TAKE(pObj)                 sMutexHandleTake(pObj)
#define MUTEX_HANDLE_GIVE(pObj)                 sMutexHandleGive(pObj)
#define MUTEX_API_TAKE(pObj)                    sMutexApiTake(pObj)
#define MUTEX_API_GIVE(pObj)                    sMutexApiGive(pObj)
#else
#define MUTEX_HANDLE_TAKE(pObj)                 (0)
#define MUTEX_HANDLE_GIVE(pObj)
#define MUTEX_API_TAKE(pObj)                    (0)
#define MUTEX_API_GIVE(pObj)
#endif

/***** local type definitions ************************************************/

typedef enum
{
    FLASHING_STATE_PROGRAMMING,
    FLASHING_STATE_DONE
} FlashingState_t;

typedef uint8_t Mutex_t;

typedef struct
{
    Sii9777Config_t                   config;
    Sii9777EventFlags_t               eventFlagsStat;
    Sii9777EventFlags_t               eventFlagsMask;
    Sii9777EventFlags_t               eventFlagsUser;
    uint32_t                          hInst;
    Sii9777EventCallbackFunc_t        siiLibEvtCbFunc;
    uint8_t                           devId;
#if( SII9777_USER_DEF__MULTI_THREAD )
    SiiPlatformSemaphore_t            semaphoreApi;
    SiiPlatformSemaphore_t            semaphoreIpc;
    SiiPlatformSemaphore_t            semaphoreHandle;
    bool_t                            bIpcGiven;
    bool_t                            bFromUserCallBack;
#endif

    uint32_t		                  flashAddr;
    uint32_t		                  flashSize;
    bool_t                            bFlashRead;
    uint8_t*                          pFlashPacket;
#if( SII9777_FLASH_BUFFER_SIZE )
    uint8_t*                          pPacketPntr;
    uint16_t                          packetSize;
#endif

    SiiDrvIpcTagCode_t                tagCode;
    Sii9777BootStat_t                 bootStat;
    SiiDrvIpcMesInst_t                mesReqInst;
    SiiDrvIpcMesInst_t                mesAckInst;
    SiiSysTimeMilli_t                 bootTimeOut;
} Obj_t;

/***** local prototypes ******************************************************/

static void sResourcesDelete( Obj_t* pObj );
static void sHandleFlashProg( Obj_t* pObj );
static void sHandleIpcInterrupts( Obj_t* pObj );
static void sHandleIspInterrupts( Obj_t* pObj );
static void sRequestMessageInit( Obj_t* pObj, SiiDrvIpcOpCode_t opCode );
static SiiRetVal_t sWaitForValidIpcResponseMessage( Obj_t* pObj, SiiDrvIpcOpCode_t opCode );
#if( SII9777_USER_DEF__MULTI_THREAD )
static SiiRetVal_t sSemaphoreIpcTake( Obj_t* pObj );
static void sSemaphoreIpcGive( Obj_t* pObj );
static SiiRetVal_t sMutexHandleTake( Obj_t* pObj );
static void sMutexHandleGive( Obj_t* pObj );
static SiiRetVal_t sMutexApiTake( Obj_t* pObj );
static void sMutexApiGive( Obj_t* pObj );
#endif
static void sDeviceRestart( Obj_t* pObj );
static bool_t sDevicePing( Obj_t* pObj );
static void sInterruptsEnableSet( Obj_t* pObj, bool_t bEnable );
static SiiRetVal_t sCbusLocalDevcapSet( Obj_t* pObj, uint8_t *pData, uint8_t offset, uint8_t len );
static SiiRetVal_t sCbusLocalDevcapGet( Obj_t* pObj, uint8_t *poData, uint8_t offset, uint8_t len );

/***** local data objects ****************************************************/

static SiiSysObjListInst_t        sObjListInst = SII_INST_NULL;
static Obj_t*                     spObj        = NULL;

/***** call-back functions ***************************************************/

void SiiIpcCallBack( SiiDrvIpcEvent_t ipcEvent )
{     
    Obj_t* pObj = spObj;

    if( ipcEvent & SII_DRV_IPC_EVENT__REQUEST )
    {
        SiiDrvIpcMesInst_t mesInst = SiiDrvIpcMessageCreate();

        SiiDrvIpcMessageReceive(mesInst);
        if( SII_DRV_IPC_TYPE__NOTIFY == SiiDrvIpcTypeGet(mesInst) )
        {
            Sii9777EventFlags_t eventFlags = (Sii9777EventFlags_t)SiiDrvIpcLongPop(mesInst);

            if( eventFlags & SII9777_EVENT_FLAGS__BOOT_DONE )
            {
                pObj->bootStat = SII9777_BOOT_STAT__SUCCESS;
            }
            pObj->eventFlagsUser |= eventFlags;
            SII_LOG1A("SII_DRV_IPC_TYPE__NOTIFY", pObj, ("\n"));
        }
        else if( SII_DRV_IPC_TYPE__RESPONSE == SiiDrvIpcTypeGet(mesInst) )
        {
            SII_LOG1A("SII_DRV_IPC_TYPE__RESPONSE", pObj, ("\n"));
            SiiDrvIpcMessageCopy(pObj->mesAckInst, mesInst);

#if( SII9777_USER_DEF__MULTI_THREAD )
            /* Don't give IPC semaphore if IPC request was initiated through Sii9777Handle */
            if( !pObj->bFromUserCallBack )
            {
                sSemaphoreIpcGive(pObj);
            }
#endif
        }
        SiiDrvIpcMessageDelete(mesInst);
    }
}

/***** public functions ******************************************************/

SiiInst_t Sii9777Create( uint8_t devId, Sii9777EventCallbackFunc_t pCallback, Sii9777Config_t* pConfig )
{
    SiiInst_t     parentInst = SII_INST_NULL;
    Obj_t*        pObj       = NULL;
    SiiRetVal_t   retVal     = 0;

    if( !sObjListInst )
    {
        sObjListInst = SiiSysObjListCreate(SII_MODULE_NAME_GET(), sizeof(Obj_t));
        if( !sObjListInst )
        {
            retVal = 1;
        }
    }

    if( !retVal )
    {
        /* Allocate memory for object */
        pObj = (Obj_t*)SiiSysObjInstanceCreate(sObjListInst, parentInst, "RX");
        if( !pObj )
        {
            /* Check for empty list */
            if( !SiiSysObjFirstGet(sObjListInst) )
            {
                /* If empty then delete list of objects */
                SiiSysObjListDelete(sObjListInst);
                sObjListInst = SII_INST_NULL;
            }
            retVal = 2;
        }
    }

    if( !retVal )
    {
        spObj = pObj;
        SII_LOG1A("Create", pObj, ("\n"));

        pObj->devId               = devId, 
        /* allocate DrvAdapt handle */
        pObj->siiLibEvtCbFunc     = pCallback;
        pObj->config              = *pConfig;
        pObj->eventFlagsStat      = 0x00000000;
        pObj->eventFlagsMask      = SII9777_EVENT_FLAGS__BOOT_DONE;
        pObj->eventFlagsUser      = 0x00000000;
        pObj->flashAddr           = 0;
        pObj->flashSize           = 0;
        pObj->bFlashRead          = true;
        pObj->pFlashPacket        = NULL;
#if( SII9777_FLASH_BUFFER_SIZE    )
        pObj->pPacketPntr         = NULL;
        pObj->packetSize          = 0;
#endif
        pObj->tagCode             = 0;
        pObj->bootStat            = SII9777_BOOT_STAT__IN_PROGRESS;
        pObj->mesReqInst          = SiiDrvIpcMessageCreate();
        pObj->mesAckInst          = SiiDrvIpcMessageCreate();
        pObj->bootTimeOut         = 0;
#if( SII9777_USER_DEF__MULTI_THREAD )
        pObj->semaphoreApi        = 0;
        pObj->semaphoreIpc        = 0;
        pObj->semaphoreHandle     = 0;
        pObj->bIpcGiven           = false;
        pObj->bFromUserCallBack   = false;
#endif
    }

#if( SII9777_USER_DEF__MULTI_THREAD )
    /* Create semaphore that functions as Mutex for making API multi-thread safe */
    if( !retVal && (SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreCreate("SemaphoreApi", 1, 1, &pObj->semaphoreApi)) )
    {
        SII_ASSERT(0);
        sResourcesDelete(pObj);
        retVal = 3;
    }

    if( !retVal && (SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreCreate("SemaphoreIpc", 1, 0, &pObj->semaphoreIpc)) )
    {
        SII_ASSERT(0);
        sResourcesDelete(pObj);
        retVal = 3;
    }

    if( !retVal && (SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreCreate("SemaphoreHandle", 1, 1, &pObj->semaphoreHandle)) )
    {
        SII_ASSERT(0);
        sResourcesDelete(pObj);
        retVal = 4;
    }
#endif

    if( !retVal )
    {
        /* initialize IPC module */
        SiiDrvIpcCreate();

        /* Enable INT signal activation */
        sInterruptsEnableSet(pObj, true);

        if( pConfig->bDeviceReset )
        {
            /* Apply device reset */
            sDeviceRestart(pObj);
        }
        else
        {
            /* Device may still be booting, set boot timeout timer */
            SiiSysTimeOutMilliSet(&pObj->bootTimeOut, BOOT_TIME_MAX);
        }

//        MUTEX_API_GIVE(pObj);
    }
    return (retVal) ? (SII_INST_NULL) : (SII_OBJ2INST(pObj));
}
 
void Sii9777Delete( SiiInst_t inst )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Delete", pObj, (""));

        /* Disable INT signal activation */
        sInterruptsEnableSet(pObj, false);

        SiiDrvIpcDelete();

        sResourcesDelete(pObj);
    }
}

SiiRetVal_t Sii9777ConfigureSet( SiiInst_t inst, Sii9777Config_t* pConfig )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ConfigureSet", pObj, ("\n"));

        /* Reset Sii9777 device */
        pObj->config = *pConfig;
        sDeviceRestart(pObj);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ConfigureGet( SiiInst_t inst, Sii9777Config_t* pConfig )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ConfigureGet", pObj, ("\n"));

        *pConfig = pObj->config;
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777Handle( SiiInst_t inst )
{
    Obj_t*              pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t         retVal = 0;
    Sii9777EventFlags_t flags  = 0;

    retVal = MUTEX_HANDLE_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777Handle", pObj, ("\n"));

        /* Temperarily disable all interrupts to force INT signal to go low */
        sInterruptsEnableSet(pObj, false);

        /* Handle IPC interrupts */
        sHandleIpcInterrupts(pObj);

        /* Handle ISP interrupts */
        sHandleIspInterrupts(pObj);

        /* Release INT force low condition to allow level triggered ISR to be invoked in case some interrupt bits still were set */
        sInterruptsEnableSet(pObj, true);

        /* Update Event status */
        pObj->eventFlagsStat |= pObj->eventFlagsUser;
        pObj->eventFlagsUser = 0x00000000;
        flags = pObj->eventFlagsStat & pObj->eventFlagsMask;
        pObj->eventFlagsStat &= (~pObj->eventFlagsMask);
    }
    
    /* Release Mutex before calling user call-back handler so that user application can call APIs from call-back handler */
    MUTEX_HANDLE_GIVE(pObj);

    if( !retVal )
    {
        if( flags )
        {
            if( pObj->siiLibEvtCbFunc )
            {
#if( SII9777_USER_DEF__MULTI_THREAD )
                /* Set flag to allow API calls indentify if they were called from user call-back */
                pObj->bFromUserCallBack = true;
#endif

                /* Call notification handler */
                pObj->siiLibEvtCbFunc(SII_OBJ2INST(pObj), flags);

#if( SII9777_USER_DEF__MULTI_THREAD )
                pObj->bFromUserCallBack = false;
#endif
            }
        }
    }
    
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777StandbySet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*              pObj    = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t         retVal  = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777StandbySet", pObj, ("%d", (uint16_t)*pbOn));

        sRequestMessageInit(pObj, SII9777_STANDBY_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_STANDBY_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777StandbyGet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*              pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t         retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777StandbyGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_STANDBY_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_STANDBY_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (uint16_t)*pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ChipIdQuery( SiiInst_t inst, uint32_t* pChipIdStat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ChipIdQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CHIPID_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CHIPID_QUERY);
        if( !retVal )
        {
            *pChipIdStat = SiiDrvIpcLongPop(pObj->mesAckInst);
            SII_LOG1B(("%08lX", *pChipIdStat));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ChipRevisionQuery( SiiInst_t inst, uint32_t* pChipRevStat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ChipRevisionQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CHIPREVISION_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CHIPREVISION_QUERY);
        if( !retVal )
        {
            *pChipRevStat = SiiDrvIpcLongPop(pObj->mesAckInst);
            SII_LOG1B(("%08lX", *pChipRevStat));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ProductIdQuery( SiiInst_t inst, uint32_t* pProdIdStat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ProductIdQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_PRODUCTID_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_PRODUCTID_QUERY);
        if( !retVal )
        {
            *pProdIdStat = SiiDrvIpcLongPop(pObj->mesAckInst);
            SII_LOG1B(("%08lX", *pProdIdStat));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777FirmwareVersionQuery( SiiInst_t inst, uint32_t* pVersionStat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777FirmwareVersionQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_FIRMWAREVERSION_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_FIRMWAREVERSION_QUERY);
        if( !retVal )
        {
            *pVersionStat = SiiDrvIpcLongPop(pObj->mesAckInst);
            SII_LOG1B(("%08lX", *pVersionStat));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ReleaseTimeStampQuery( SiiInst_t inst, uint32_t* pTimeStamp )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ReleaseTimeStampQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_RELEASETIMESTAMP_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_RELEASETIMESTAMP_QUERY);
        if( !retVal )
        {
            *pTimeStamp = SiiDrvIpcLongPop(pObj->mesAckInst);
            SII_LOG1B(("%08lX", *pTimeStamp));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777BootStatusQuery( SiiInst_t inst, Sii9777BootStat_t* pBootStat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;
    
    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777BootStatusQuery", pObj, (""));

        if( SII9777_BOOT_STAT__IN_PROGRESS == pObj->bootStat )
        {
            /* Ping device */
            if( sDevicePing(pObj) )
            {
                pObj->bootStat = SII9777_BOOT_STAT__SUCCESS;
            }
            else
            {
                if( SiiSysTimeOutMilliIs(&pObj->bootTimeOut) )
                {
                    pObj->bootStat = SII9777_BOOT_STAT__FAILURE;
                }
            }
        }

        *pBootStat = pObj->bootStat;
        SII_LOG1B(("%d", (uint16_t)*pBootStat));
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EventFlagsMaskSet( SiiInst_t inst, Sii9777EventFlags_t* pFlags )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    
    if( !retVal )
    {
        SII_LOG1A("Sii9777EventFlagsMaskSet", pObj, ("%08lX", *pFlags));

        if( SII9777_BOOT_STAT__SUCCESS == pObj->bootStat )
        {
            sRequestMessageInit(pObj, SII9777_EVENTFLAGSMASK_SET);
            SiiDrvIpcLongPush(pObj->mesReqInst, (*pFlags) & EVENT_MASK_EXTERNAL);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EVENTFLAGSMASK_SET);
            if( !retVal )
            {
                pObj->eventFlagsMask =  *pFlags;
            }
        }
        else
        {
            pObj->eventFlagsMask = (*pFlags) & EVENT_MASK_INTERNAL;
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EventFlagsMaskGet( SiiInst_t inst, Sii9777EventFlags_t* pFlags )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EventFlagsMaskGet", pObj, (""));

        *pFlags = pObj->eventFlagsMask;
        SII_LOG1B(("%08lX", *pFlags));
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EventFlagsQuery( SiiInst_t inst, Sii9777EventFlags_t* pFlags )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EventFlagsQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_EVENTFLAGS_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EVENTFLAGS_QUERY);
        if( !retVal )
        {
            *pFlags = (SiiDrvIpcLongPop(pObj->mesAckInst) & EVENT_MASK_EXTERNAL) | (pObj->eventFlagsStat & EVENT_MASK_INTERNAL);
            SII_LOG1B(("%08lX", *pFlags));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EventFlagsStatusClear( SiiInst_t inst, Sii9777EventFlags_t* pFlags )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EventFlagsStatusClear", pObj, ("%08lX", *pFlags));

        pObj->eventFlagsStat &= (~(*pFlags));
        sRequestMessageInit(pObj, SII9777_EVENTFLAGSSTATUS_CLEAR);
        SiiDrvIpcLongPush(pObj->mesReqInst, *pFlags);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EVENTFLAGSSTATUS_CLEAR);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777GpioConfigSet( SiiInst_t inst, uint16_t *pGpioPins )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777GpioConfigSet", pObj, ("%04X", *pGpioPins));

        sRequestMessageInit(pObj, SII9777_GPIOCONFIG_SET);
        SiiDrvIpcWordPush(pObj->mesReqInst, *pGpioPins);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_GPIOCONFIG_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777GpioConfigGet( SiiInst_t inst, uint16_t *pGpioPins )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777GpioConfigGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_GPIOCONFIG_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_GPIOCONFIG_GET);
        if( !retVal )
        {
            *pGpioPins = SiiDrvIpcWordPop(pObj->mesAckInst);
            SII_LOG1B(("%04X", *pGpioPins));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777GpioOutputSet( SiiInst_t inst, uint16_t* pGpioPins )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777GpioOutputSet", pObj, ("%04X", *pGpioPins));

        sRequestMessageInit(pObj, SII9777_GPIOOUTPUT_SET);
        SiiDrvIpcWordPush(pObj->mesReqInst, *pGpioPins);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_GPIOOUTPUT_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777GpioOutputClr( SiiInst_t inst, uint16_t* pGpioPins )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777GpioOutputClr", pObj, ("%04X", *pGpioPins));

        sRequestMessageInit(pObj, SII9777_GPIOOUTPUT_CLR);
        SiiDrvIpcWordPush(pObj->mesReqInst, *pGpioPins);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_GPIOOUTPUT_CLR);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777GpioQuery( SiiInst_t inst, uint16_t* pGpioPins )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777GpioQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_GPIO_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_GPIO_QUERY);
        if( !retVal )
        {
            *pGpioPins = SiiDrvIpcWordPop(pObj->mesAckInst);
            SII_LOG1B(("%04X", *pGpioPins));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777GpioEventMaskSet( SiiInst_t inst, uint16_t*  pGpioPins )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777GpioEventMaskSet", pObj, ("%04X", *pGpioPins));

        sRequestMessageInit(pObj, SII9777_GPIOEVENTMASK_SET);
        SiiDrvIpcWordPush(pObj->mesReqInst, *pGpioPins);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_GPIOEVENTMASK_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777GpioEventMaskGet( SiiInst_t inst, uint16_t*  pGpioPins )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777GpioEventMaskGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_GPIOEVENTMASK_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_GPIOEVENTMASK_GET);
        if( !retVal )
        {
            *pGpioPins = SiiDrvIpcWordPop(pObj->mesAckInst);
            SII_LOG1B(("%04X", *pGpioPins));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

/* Obsoleted */
SiiRetVal_t Sii9777FirmwareInit( SiiInst_t inst )
{
    return Sii9777FlashInit(inst, SII9777_FLASH_OPERATION__WRITE, SII9777_FLASH_TARGET__FULL_DEVICE);
}

SiiRetVal_t Sii9777FlashInit( SiiInst_t inst, Sii9777FlashOperation_t operation, Sii9777FlashTarget_t target )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777FlashInit", pObj, (""));

        /* Check flash Operation request code */
        switch( operation )
        {
            case SII9777_FLASH_OPERATION__READ :
                pObj->bFlashRead   = true;
                SiiDrvIspReadInit();
                break;

            case SII9777_FLASH_OPERATION__WRITE :
                pObj->bFlashRead   = false;
                break;

            case SII9777_FLASH_OPERATION__WRITE_VERIFY :
                pObj->bFlashRead   = false;
                break;

            default: retVal = 1; break;
        }

        if( !retVal )
        {
            switch( target )
            {
                case SII9777_FLASH_TARGET__FULL_DEVICE:
                    pObj->flashAddr    = 0x00000;
                    pObj->flashSize    = 0x80000; /* Full size of 4Mbit flash */
                    if( SII9777_FLASH_OPERATION__READ != operation )
                    {
                        SiiDrvIspChipErase();
                    }
                    break;

                case SII9777_FLASH_TARGET__EDID_TABLES:
                    pObj->flashAddr    = 0x05000;
                    pObj->flashSize    = 0x00400; /* Size of power-up default EDID tables */
                    if( SII9777_FLASH_OPERATION__READ != operation )
                    {
                        SiiDrvIspSectorErase(0x05000);
                    }
                    break;

                case SII9777_FLASH_TARGET__FIRMWARE:
                default: retVal = 1; break;
            }
        }

        if( !retVal )
        {
#if( SII9777_FLASH_BUFFER_SIZE )
            if( pObj->pFlashPacket )
            {
                 SiiSysMallocDelete(pObj->pFlashPacket);
            }
#endif
            pObj->pFlashPacket = NULL;
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777FirmwareUpdate( SiiInst_t inst, const uint8_t* pData, uint16_t length )
{
    return Sii9777FlashUpdate(inst, (uint8_t *)pData, length);
}

SiiRetVal_t Sii9777FlashUpdate( SiiInst_t inst, uint8_t* pData, uint16_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777FlashUpdate", pObj, ("length=%d\n", length));

        if( NULL == pData )
        {
            retVal = 1;
        }
        else if( 0 == length )
        {
            retVal = 2;
        }
        else if( 0 == pObj->flashSize )
        {
            /* Flash is already programmed, don't expect more data */
            retVal = 3;
        }
        else
        {
            if( pObj->bFlashRead )
            {
                SiiDrvIspBurstRead(pObj->flashAddr, pData, length);
                pObj->flashAddr += length;
            }
            else /* Writing Flash */
            {
                if( pObj->pFlashPacket )
                {
                    /* Previous buffer has not been emptied */
                    retVal = 4;
                }

#if( SII9777_FLASH_BUFFER_SIZE )
                if( !retVal && (length > SII9777_FLASH_BUFFER_SIZE) )
                {
                    /* Rquested packet size is bigger than maximum allowed packet size */
                    retVal = 5;
                }
#endif

#if( !SII9777_FLASH_BUFFER_SIZE )
                if( !retVal && (length != 256) )
                {
                    /* Rquested packet size is bigger than maximum allowed packet size */
                    retVal = 6;
                }
#endif

                if( !retVal )
                {
#if( SII9777_FLASH_BUFFER_SIZE )
                     /* Create data flash packet */
                     pObj->pFlashPacket = (uint8_t*)SiiSysMallocCreate(length);
                     SII_ASSERT(pObj->pFlashPacket);

                     if( pObj->pFlashPacket )
                     {
                         /* Copy data into flash packet */
                         SII_MEMCPY(pObj->pFlashPacket, pData, length);

                         /* Initialize packet pointer and size */
                         pObj->pPacketPntr  = pObj->pFlashPacket;
                         pObj->packetSize   = length;

                         /* Start flashing packet (interrupt driven) */
                         sHandleFlashProg(pObj);
                     }
                     else
                     {
                         retVal = 7;
                     }
#else
                    pObj->pFlashPacket = (uint8_t*)pData;

                    /* Start flashing packet (interrupt driven) */
                    sHandleFlashProg(pObj);
#endif
                }
            }

            if( !retVal )
            {
                /* Update total number of bytes programmed */
                if( pObj->flashSize < length )
                {
                    pObj->flashSize = 0;
                }
                else
                {
                    pObj->flashSize -= length;
                }
            }
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777FirmwareStatusQuery( SiiInst_t inst, Sii9777FlashStat_t* poStatus )
{
    return Sii9777FlashStatusQuery(inst, poStatus);
}

SiiRetVal_t Sii9777FlashStatusQuery( SiiInst_t inst, Sii9777FlashStat_t* poStatus )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777FlashStatusQuery", pObj, (""));

        if( pObj->bFlashRead || SiiDrvIspOperationDone() )
        {
            *poStatus = SII9777_FIRMWARE_STAT__READY;
        }
        else
        {
            *poStatus = SII9777_FIRMWARE_STAT__IN_PROGRESS;
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777FirmwareRestart( SiiInst_t inst )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777FirmwareRestart", pObj, (""));

        sDeviceRestart(pObj);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkModeSet( SiiInst_t inst, Sii9777DualLinkMode_t* pDualLinkMode )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkModeSet", pObj, ("%d", *pDualLinkMode));

        sRequestMessageInit(pObj, SII9777_DUALLINKMODE_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, *pDualLinkMode);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKMODE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkModeGet( SiiInst_t inst, Sii9777DualLinkMode_t* pDualLinkMode )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkModeGet", pObj, (""));

        sRequestMessageInit(pObj, Sii9777_DUALLINKMODE_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, Sii9777_DUALLINKMODE_GET);
        if( !retVal )
        {
            *pDualLinkMode = (Sii9777DualLinkMode_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pDualLinkMode));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777Plus5vQuery( SiiInst_t inst, Sii9777RxPort_t port, bool_t *pbPlus5v )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777Plus5vQuery", pObj, ("port=%d : ", (uint16_t)port));

        sRequestMessageInit(pObj, SII9777_PLUS5V_QUERY);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_PLUS5V_QUERY);
        if( !retVal )
        {
            *pbPlus5v = (Sii9777DualLinkMode_t)SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbPlus5v));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777InputSelectSet( SiiInst_t inst, Sii9777RxPort_t* pPort )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777InputSelectSet", pObj, ("%d", *pPort));

        sRequestMessageInit(pObj, SII9777_INPUTSELECT_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, *pPort);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_INPUTSELECT_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777InputSelectGet( SiiInst_t inst, Sii9777RxPort_t* pPort )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777InputSelectGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_INPUTSELECT_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_INPUTSELECT_GET);
        if( !retVal )
        {
            *pPort = (Sii9777RxPort_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pPort));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EdidDdcSet( SiiInst_t inst, Sii9777RxPort_t port, bool_t* pbDdc )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EdidDdcSet", pObj, ("port=%d : %d", (uint16_t)port, (uint16_t)*pbDdc));

        sRequestMessageInit(pObj, SII9777_EDIDDDC_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbDdc);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EDIDDDC_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EdidDdcGet( SiiInst_t inst, Sii9777RxPort_t port, bool_t* pbDdc )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EdidDdcGet", pObj, ("port=%d : ", (uint16_t)port));

        sRequestMessageInit(pObj, SII9777_EDIDDDC_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EDIDDDC_GET);
        if( !retVal )
        {
            *pbDdc = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbDdc));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EdidReplicateEnableSet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EdidReplicateEnableSet", pObj, ("%d", *pbOn));

        sRequestMessageInit(pObj, SII9777_EDIDREPLICATE_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EDIDREPLICATE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EdidReplicateEnableGet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EdidReplicateEnableGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_EDIDREPLICATE_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EDIDREPLICATE_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EdidSet( SiiInst_t inst, Sii9777RxPort_t port, uint16_t offset, uint8_t *pData, uint16_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EdidSet", pObj, ("port=%d, offset=%04X, length=%d\n", (uint16_t)port, offset, length));
        if( !retVal && (255 < offset) )
        {
            retVal = 40;
        }
        if( !retVal && (256 < (length+offset)) )
        {
            retVal = 50;
        }
        if( !retVal )
        {
            SII_LOG1B_ARRAY(pData, length);

            sRequestMessageInit(pObj, SII9777_EDID_SET);
            SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
            SiiDrvIpcWordPush(pObj->mesReqInst, offset);
            SiiDrvIpcWordPush(pObj->mesReqInst, length);
            SiiDrvIpcArrayPush(pObj->mesReqInst, pData, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EDID_SET);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EdidGet( SiiInst_t inst, Sii9777RxPort_t port, uint16_t offset, uint8_t *pData, uint16_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EdidSet", pObj, ("port=%d, offset=%04X, length=%d\n", (uint16_t)port, offset, length));
        if( !retVal && (255 < offset) )
        {
            retVal = 40;
        }
        if( !retVal && (256 < (length+offset)) )
        {
            retVal = 50;
        }
        if( !retVal )
        {
            sRequestMessageInit(pObj, SII9777_EDID_GET);
            SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
            SiiDrvIpcWordPush(pObj->mesReqInst, offset);
            SiiDrvIpcWordPush(pObj->mesReqInst, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EDID_GET);
        }
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pData, length);
            SII_LOG1B_ARRAY(pData, length);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777EdidQuery( SiiInst_t inst, Sii9777RxPort_t port, uint16_t offset, uint8_t *pData, uint16_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777EdidQuery", pObj, ("port=%d, offset=%04X, length=%d\n", (uint16_t)port, offset, length));

        sRequestMessageInit(pObj, SII9777_EDID_QUERY);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcWordPush(pObj->mesReqInst, offset);
        SiiDrvIpcWordPush(pObj->mesReqInst, length);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_EDID_QUERY);
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pData, length);
            SII_LOG1B_ARRAY(pData, length);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t	Sii9777HpdReplicateEnableSet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HpdReplicateEnableSet", pObj, ("%d", *pbOn));

        sRequestMessageInit(pObj, SII9777_HPDREPLICATEENABLE_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HPDREPLICATEENABLE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t	Sii9777HpdReplicateEnableGet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HpdReplicateEnableGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_HPDREPLICATEENABLE_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HPDREPLICATEENABLE_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HpdSet( SiiInst_t inst, Sii9777RxPort_t port, bool_t *pbHpd )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HpdReplicateEnableSet", pObj, ("port=%d : %d", (uint16_t)port, (uint16_t)*pbHpd));

        sRequestMessageInit(pObj, SII9777_HPD_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbHpd);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HPD_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HpdGet( SiiInst_t inst, Sii9777RxPort_t port, bool_t *pbHpd )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HpdGet", pObj, ("port=%d : ", (uint16_t)port));

        sRequestMessageInit(pObj, SII9777_HPD_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HPD_GET);
        if( !retVal )
        {
            *pbHpd = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (uint16_t)*pbHpd));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HpdQuery( SiiInst_t inst, Sii9777RxPort_t port, bool_t *pbHpd )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HpdQuery", pObj, ("port=%d : ", (uint16_t)port));

        sRequestMessageInit(pObj, SII9777_HPD_QUERY);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HPD_QUERY);
        if( !retVal )
        {
            *pbHpd = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbHpd));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777AvLinkQuery( SiiInst_t inst, Sii9777AvLink_t *pAvLink )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777AvLinkQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_AVLINK_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_AVLINK_QUERY);
        if( !retVal )
        {
            *pAvLink = (Sii9777AvLink_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pAvLink));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777AvMuteQuery( SiiInst_t inst, bool_t *pbAvMute )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777AvMuteQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_AVMUTE_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_AVMUTE_QUERY);
        if( !retVal )
        {
            *pbAvMute = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbAvMute));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777VideoTimingQuery( SiiInst_t inst, Sii9777VideoTiming_t *pVideoTiming )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777VideoTimingQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_VIDEOTIMING_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_VIDEOTIMING_QUERY);
        if( !retVal )
        {
            pVideoTiming->interlaced  = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pVideoTiming->posHsync    = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pVideoTiming->posVsync    = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pVideoTiming->pixelFreq   = SiiDrvIpcLongPop(pObj->mesAckInst);
            pVideoTiming->vTotal      = SiiDrvIpcWordPop(pObj->mesAckInst);
            pVideoTiming->vActive     = SiiDrvIpcWordPop(pObj->mesAckInst);
            pVideoTiming->vSync       = SiiDrvIpcWordPop(pObj->mesAckInst);
            pVideoTiming->vFrontPorch = SiiDrvIpcWordPop(pObj->mesAckInst);
            pVideoTiming->hTotal      = SiiDrvIpcWordPop(pObj->mesAckInst);
            pVideoTiming->hActive     = SiiDrvIpcWordPop(pObj->mesAckInst);
            pVideoTiming->hSync       = SiiDrvIpcWordPop(pObj->mesAckInst);
            pVideoTiming->hFrontPorch = SiiDrvIpcWordPop(pObj->mesAckInst);

            SII_LOG1B(("interlaced  = %d\n",  pVideoTiming->interlaced));
            SII_LOG1B(("posHsync    = %d\n",  pVideoTiming->posHsync));
            SII_LOG1B(("posVsync    = %d\n",  pVideoTiming->posVsync));
            SII_LOG1B(("pixelFreq   = %ld\n", pVideoTiming->pixelFreq));
            SII_LOG1B(("vTotal      = %d\n",  pVideoTiming->vTotal));
            SII_LOG1B(("vActive     = %d\n",  pVideoTiming->vActive));
            SII_LOG1B(("vSync       = %d\n",  pVideoTiming->vSync));
            SII_LOG1B(("vFrontPorch = %d\n",  pVideoTiming->vFrontPorch));
            SII_LOG1B(("hTotal      = %d\n",  pVideoTiming->hTotal));
            SII_LOG1B(("hActive     = %d\n",  pVideoTiming->hActive));
            SII_LOG1B(("hSync       = %d\n",  pVideoTiming->hSync));
            SII_LOG1B(("hFrontPorch = %d\n",  pVideoTiming->hFrontPorch));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777VideoFormatQuery( SiiInst_t inst, Sii9777VideoFmt_t *poVideoFmt )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777VideoFormatQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_VIDEOFMT_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_VIDEOFMT_QUERY);
        if( !retVal )
        {
            poVideoFmt->vidRes              = (Sii9777VidRes_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->mode3D              = (Sii97773dMode_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->pictureAR           = (Sii9777PictAR_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->pixelRepetition     = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->pixFmt.clrFmt       = (Sii9777ClrFmt_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->pixFmt.crSmpl       = (Sii9777CrSmpl_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->pixFmt.bitDepth     = (Sii9777BitDepth_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->activeAR            = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->picScaling          = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->scanInfo            = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->contentType         = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->isITContent         = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->isActiveFormat      = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->isVertBarInfoValid  = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->isHorizBarInfoValid = SiiDrvIpcBytePop(pObj->mesAckInst);
            poVideoFmt->vertBarInfo.lineNumEndOfTopBar      = SiiDrvIpcWordPop(pObj->mesAckInst);
            poVideoFmt->vertBarInfo.lineNumStartOfBottomBar = SiiDrvIpcWordPop(pObj->mesAckInst);
            poVideoFmt->horizBarInfo.pixNumEndOfLeftBar     = SiiDrvIpcWordPop(pObj->mesAckInst);
            poVideoFmt->horizBarInfo.pixNumStartOfRightBar  = SiiDrvIpcWordPop(pObj->mesAckInst);

            SII_LOG1B(("vidRes              = %d\n", (uint16_t)poVideoFmt->vidRes));
            SII_LOG1B(("mode3D              = %d\n", (uint16_t)poVideoFmt->mode3D));
            SII_LOG1B(("pictureAR           = %d\n", (uint16_t)poVideoFmt->pictureAR));
            SII_LOG1B(("pixelRepetition     = %d\n", (uint16_t)poVideoFmt->pixelRepetition));
            SII_LOG1B(("pixFmt.clrFmt       = %d\n", (uint16_t)poVideoFmt->pixFmt.clrFmt));
            SII_LOG1B(("pixFmt.crSmpl       = %d\n", (uint16_t)poVideoFmt->pixFmt.crSmpl));
            SII_LOG1B(("pixFmt.bitDepth     = %d\n", (uint16_t)poVideoFmt->pixFmt.bitDepth));
            SII_LOG1B(("activeAR            = %d\n", (uint16_t)poVideoFmt->activeAR));
            SII_LOG1B(("picScaling          = %d\n", (uint16_t)poVideoFmt->picScaling));
            SII_LOG1B(("scanInfo            = %d\n", (uint16_t)poVideoFmt->scanInfo));
            SII_LOG1B(("contentType         = %d\n", (uint16_t)poVideoFmt->contentType));
            SII_LOG1B(("isITContent         = %d\n", (uint16_t)poVideoFmt->isITContent));
            SII_LOG1B(("isActiveFormat      = %d\n", (uint16_t)poVideoFmt->isActiveFormat));
            SII_LOG1B(("isVertBarInfoValid  = %d\n", (uint16_t)poVideoFmt->isVertBarInfoValid));
            SII_LOG1B(("isHorizBarInfoValid = %d\n", (uint16_t)poVideoFmt->isHorizBarInfoValid));
            SII_LOG1B(("vertBarInfo.lineNumEndOfTopBar      = %d\n", (uint16_t)poVideoFmt->vertBarInfo.lineNumEndOfTopBar));
            SII_LOG1B(("vertBarInfo.lineNumStartOfBottomBar = %d\n", (uint16_t)poVideoFmt->vertBarInfo.lineNumStartOfBottomBar));
            SII_LOG1B(("horizBarInfo.pixNumEndOfLeftBar     = %d\n", (uint16_t)poVideoFmt->horizBarInfo.pixNumEndOfLeftBar));
            SII_LOG1B(("horizBarInfo.pixNumStartOfRightBar  = %d\n", (uint16_t)poVideoFmt->horizBarInfo.pixNumStartOfRightBar));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777AudioFormatQuery( SiiInst_t inst, Sii9777AudioFmt_t *pAudioFmt )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777AudioFormatQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_AUDIOFMT_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_AUDIOFMT_QUERY);
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pAudioFmt->aifData, 10);
            SiiDrvIpcArrayPop(pObj->mesAckInst, pAudioFmt->csData, 7);

            SII_LOG1B(("Audio Info Frame : "));
            SII_LOG1B_ARRAY(pAudioFmt->aifData, 10);
            SII_LOG1B(("channel status   : "));
            SII_LOG1B_ARRAY(pAudioFmt->csData, 7);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777IsrcQuery( SiiInst_t inst, Sii9777Isrc_t* pIsrc )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777IsrcQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_ISRC_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_ISRC_QUERY);
        if( !retVal )
        {
            pIsrc->isrcCont   = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pIsrc->isrcStatus = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pIsrc->isrcValid  = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SiiDrvIpcArrayPop(pObj->mesAckInst, pIsrc->isrcData, 32);

            SII_LOG1B(("isrcCont   = %02X ", (uint16_t)pIsrc->isrcCont));
            SII_LOG1B(("isrcStatus = %02X ", (uint16_t)pIsrc->isrcStatus));
            SII_LOG1B(("isrcValid  = %02X ", (uint16_t)pIsrc->isrcValid));
            SII_LOG1B_ARRAY(pIsrc->isrcData, 32);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777AcpQuery( SiiInst_t inst, Sii9777Acp_t* pAcp )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777AcpQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_ACP_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_ACP_QUERY);
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pAcp->acpData, sizeof(pAcp->acpData));
            SII_LOG1B_ARRAY(pAcp->acpData, sizeof(pAcp->acpData));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777SpdQuery( SiiInst_t inst, Sii9777Spd_t* pSpd )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777SpdQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_SPD_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_SPD_QUERY);
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pSpd->vendorName, 8);
            SiiDrvIpcArrayPop(pObj->mesAckInst, pSpd->productDescription, 16);
            pSpd->sourceDevInfo = (Sii9777SrcDevInfo_t)SiiDrvIpcBytePop(pObj->mesAckInst);

            SII_LOG1B(("vendorName         : "));
            SII_LOG1B_ARRAY(pSpd->vendorName, 8);
            SII_LOG1B(("productDescription : "));
            SII_LOG1B_ARRAY(pSpd->productDescription, 16);
            SII_LOG1B(("sourceDevInfo      : %02X\n", pSpd->sourceDevInfo));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpDdcSet( SiiInst_t inst, Sii9777RxPort_t port, bool_t* pbDdc )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpDdcSet", pObj, ("port=%d : %d", (uint16_t)port, (uint16_t)*pbDdc));

        sRequestMessageInit(pObj, SII9777_HDCPDDC_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbDdc);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPDDC_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpDdcGet( SiiInst_t inst, Sii9777RxPort_t port, bool_t* pbDdc )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpDdcGet", pObj, ("port=%d : ", (uint16_t)port));

        sRequestMessageInit(pObj, SII9777_HDCPDDC_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPDDC_GET);
        if( !retVal )
        {
            *pbDdc = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (uint16_t)*pbDdc));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpStreamManageInfoQuery( SiiInst_t inst, Sii9777HdcpStreamManageInfo_t* pStreamManageInfo )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpStreamManageInfoQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_HDCPSTREAMMANAGEINFO_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPSTREAMMANAGEINFO_QUERY);
        if( !retVal )
        {
            pStreamManageInfo->seqNumM         = SiiDrvIpcLongPop(pObj->mesAckInst);
            pStreamManageInfo->k               = SiiDrvIpcWordPop(pObj->mesAckInst);
            pStreamManageInfo->streamIdType[0] = SiiDrvIpcWordPop(pObj->mesAckInst);
            pStreamManageInfo->streamIdType[1] = SiiDrvIpcWordPop(pObj->mesAckInst);
            pStreamManageInfo->streamIdType[2] = SiiDrvIpcWordPop(pObj->mesAckInst);
            pStreamManageInfo->streamIdType[3] = SiiDrvIpcWordPop(pObj->mesAckInst);

            SII_LOG1B(("seqNumM= %08X\n", pStreamManageInfo->seqNumM));
            SII_LOG1B(("k      = %04X\n", pStreamManageInfo->k));
            SII_LOG1B_ARRAY((uint8_t *)pStreamManageInfo->streamIdType, sizeof(pStreamManageInfo->streamIdType));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpTopologySet( SiiInst_t inst, Sii9777HdcpTopology_t *pTopology )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpTopologySet", pObj, ("\n"));
        SII_LOG1B(("dcp1DeviceDs       = %02X\n", pTopology->bHdcp1DeviceDs));
        SII_LOG1B(("bHdcp20RepeaterDs  = %02X\n", pTopology->bHdcp20RepeaterDs));
        SII_LOG1B(("MaxCascadeExceeded = %02X\n", pTopology->bMaxCascadeExceeded));
        SII_LOG1B(("MaxDevsExceeded    = %02X\n", pTopology->bMaxDevsExceeded));
        SII_LOG1B(("deviceCount        = %02X\n", pTopology->deviceCount));
        SII_LOG1B(("depth              = %02X\n", pTopology->depth));
        SII_LOG1B(("seqNumV            = %08X\n", pTopology->seqNumV));

        sRequestMessageInit(pObj, SII9777_HDCPTOPOLOGY_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, (uint8_t)pTopology->bHdcp1DeviceDs);
        SiiDrvIpcBoolPush(pObj->mesReqInst, (uint8_t)pTopology->bHdcp20RepeaterDs);
        SiiDrvIpcBoolPush(pObj->mesReqInst, (uint8_t)pTopology->bMaxCascadeExceeded);
        SiiDrvIpcBoolPush(pObj->mesReqInst, (uint8_t)pTopology->bMaxDevsExceeded);
        SiiDrvIpcBytePush(pObj->mesReqInst, pTopology->deviceCount);
        SiiDrvIpcBytePush(pObj->mesReqInst, pTopology->depth);
        SiiDrvIpcLongPush(pObj->mesReqInst, pTopology->seqNumV);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPTOPOLOGY_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpTopologyGet( SiiInst_t inst, Sii9777HdcpTopology_t *pTopology )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpTopologyGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_HDCPTOPOLOGY_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPTOPOLOGY_GET);
        if( !retVal )
        {
            pTopology->bHdcp1DeviceDs      = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pTopology->bHdcp20RepeaterDs   = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pTopology->bMaxCascadeExceeded = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pTopology->bMaxDevsExceeded    = SiiDrvIpcBoolPop(pObj->mesAckInst);
            pTopology->deviceCount         = SiiDrvIpcBytePop(pObj->mesAckInst);
            pTopology->depth               = SiiDrvIpcBytePop(pObj->mesAckInst);
            pTopology->seqNumV             = SiiDrvIpcLongPop(pObj->mesAckInst);

            SII_LOG1B(("dcp1DeviceDs       = %02X\n", pTopology->bHdcp1DeviceDs));
            SII_LOG1B(("bHdcp20RepeaterDs  = %02X\n", pTopology->bHdcp20RepeaterDs));
            SII_LOG1B(("MaxCascadeExceeded = %02X\n", pTopology->bMaxCascadeExceeded));
            SII_LOG1B(("MaxDevsExceeded    = %02X\n", pTopology->bMaxDevsExceeded));
            SII_LOG1B(("deviceCount        = %02X\n", pTopology->deviceCount));
            SII_LOG1B(("depth              = %02X\n", pTopology->depth));
            SII_LOG1B(("seqNumV            = %08X\n", pTopology->seqNumV));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpBksvRxidListSet( SiiInst_t inst, Sii9777HdcpKsv_t* pBksvList, uint8_t len )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpBksvRxidListSet", pObj, ("\n"));

        if( !pBksvList )
        {
            retVal = 1;
        }
        if( !retVal )
        {
            sRequestMessageInit(pObj, SII9777_HDCPBKSVRXIDLIST_SET);
            SiiDrvIpcBytePush(pObj->mesReqInst, len);
            while( len-- )
            {
                SiiDrvIpcArrayPush(pObj->mesReqInst, &pBksvList->b[0], 5);
                SII_LOG1B_ARRAY(&pBksvList->b[0], 5);
                pBksvList++;
             }
             SiiDrvIpcMessageSend(pObj->mesReqInst);

             retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPBKSVRXIDLIST_SET);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpBksvRxidListGet( SiiInst_t inst, Sii9777HdcpKsv_t* pBksvList, uint8_t len )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpBksvRxidListGet", pObj, (""));

        if( !pBksvList )
        {
            retVal = 1;
        }
        if( !retVal )
        {
            sRequestMessageInit(pObj, SII9777_HDCPBKSVRXIDLIST_GET);
            SiiDrvIpcBytePush(pObj->mesReqInst, len);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPBKSVRXIDLIST_GET);
        }
        if( !retVal )
        {
            uint8_t length = SiiDrvIpcBytePop(pObj->mesAckInst);

            if( length > len )
            {
                length = len;
            }

            while( length-- )
            {
                SiiDrvIpcArrayPop(pObj->mesAckInst, &pBksvList->b[0], 5);
                SII_LOG1B_ARRAY( &pBksvList->b[0], 5);
                pBksvList++;
            }
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpRepeaterEnableSet( SiiInst_t inst, bool_t *pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpRepeaterEnableSet", pObj, ("%d", (uint16_t)*pbOn));

        sRequestMessageInit(pObj, SII9777_HDCPREPEATERENABLE_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPREPEATERENABLE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpRepeaterEnableGet( SiiInst_t inst, bool_t *pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpRepeaterEnableGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_HDCPREPEATERENABLE_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPREPEATERENABLE_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (uint16_t)*pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777RxHdcpBksvOwnQuery( SiiInst_t inst, Sii9777HdcpKsv_t* pBksv )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777RxHdcpBksvOwnQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_RXHDCPBKSVOWN_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_RXHDCPBKSVOWN_QUERY);
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, (uint8_t*)pBksv, 5);
            SII_LOG1B_ARRAY((uint8_t*)pBksv, 5);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777RxHdcpRxIdOwnQuery( SiiInst_t inst, Sii9777HdcpKsv_t* pRxId)
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777RxHdcpRxIdOwnQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_RXHDCPRXIDOWN_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_RXHDCPRXIDOWN_QUERY);
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, (uint8_t*)pRxId, 5);
            SII_LOG1B_ARRAY((uint8_t*)pRxId, 5);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777HdcpStatusQuery( SiiInst_t inst, Sii9777HdcpStatus_t *pHdcpStat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777HdcpStatusQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_HDCPSTATUS_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_HDCPSTATUS_QUERY);
        if( !retVal )
        {
            *pHdcpStat = (Sii9777HdcpStatus_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d\n", (uint16_t)*pHdcpStat));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777RxCdSenseQuery( SiiInst_t inst, Sii9777CdSense_t *pCdSense )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777RxCdSenseQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CDSENSE_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CDSENSE_QUERY);
        if( !retVal )
        {
            *pCdSense = (Sii9777CdSense_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d\n", (uint16_t)*pCdSense));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777MHLVersionQuery(SiiInst_t inst, Sii9777MhlVersion_t *pMhlVersion)
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777MHLVersionQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_MHLVERSION_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_MHLVERSION_QUERY);
        if( !retVal )
        {
            *pMhlVersion = (Sii9777MhlVersion_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d\n", (uint16_t)*pMhlVersion));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusModeQuery(SiiInst_t inst, Sii9777CbusMode_t *pCbusMode)
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusModeQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CBUSMODE_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSMODE_QUERY);
        if( !retVal )
        {
            *pCbusMode = (Sii9777CbusMode_t)SiiDrvIpcBytePop(pObj->mesAckInst);      
            SII_LOG1B(("%d\n", (uint16_t)*pCbusMode));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777VbusRequestQuery(SiiInst_t inst, Sii9777VbusReq_t *pRequest)
{    
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777VbusRequestQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_VBUSREQUEST_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_VBUSREQUEST_QUERY);
        if( !retVal )
        {
            *pRequest = (Sii9777VbusReq_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d\n", (uint16_t)*pRequest));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777VbusRequestGrant(SiiInst_t inst)
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777VbusRequestGrant", pObj, (""));

        sRequestMessageInit(pObj, SII9777_VBUSREQUESTGRANT);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_VBUSREQUESTGRANT);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapSet( SiiInst_t inst, uint8_t *pData, uint8_t offset, uint8_t len )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalDevcapSet", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)len));
        if( ((offset + len) > CBUS_DEVCAP_LEN) || (len == 0) || !pData )
        {
            SII_LOG1B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }
        if( !retVal )
        {
            retVal = sCbusLocalDevcapSet(pObj, pData, offset, len);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapGet( SiiInst_t inst, uint8_t *poData, uint8_t offset, uint8_t len )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalDevcapGet", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)len));
        if( ((offset + len) > CBUS_DEVCAP_LEN) || (len == 0) || !poData )
        {
            SII_LOG1B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }
        if( !retVal )
        {
            retVal = sCbusLocalDevcapGet(pObj, poData, offset, len);
        }
        if( !retVal )
        {
            SII_LOG1B_ARRAY((uint8_t*)poData, len);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapPowSet( SiiInst_t inst, Sii9777DevCapVbusPow_t *pCbusPow )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        uint8_t dcap;

        SII_LOG1A("Sii9777CbusLocalDevcapPowSet", pObj, ("%d\n", (uint16_t)*pCbusPow));

        retVal = sCbusLocalDevcapGet(pObj, &dcap, SII9777_DEVCAP_OFFSET__DEV_CAT, 1);
        if( !retVal )
        {
            if( *pCbusPow == SII9777_VBUS_PROVIDE_POWER )
            {
                dcap |= BIT4_MASK;
            }
            else
            {
                dcap &= ~BIT4_MASK;
            }
            retVal = sCbusLocalDevcapSet(pObj, &dcap, SII9777_DEVCAP_OFFSET__DEV_CAT, 1);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapPowGet( SiiInst_t inst, Sii9777DevCapVbusPow_t *pCbusPow )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        uint8_t dcap;

        SII_LOG1A("Sii9777CbusLocalDevcapPowGet", pObj, (""));

        retVal = sCbusLocalDevcapGet(pObj, &dcap, SII9777_DEVCAP_OFFSET__DEV_CAT, 1);
        if( !retVal )
        {
            if( dcap & BIT4_MASK )
            {
                *pCbusPow = SII9777_VBUS_PROVIDE_POWER;
            }
            else
            {
                *pCbusPow = SII9777_VBUS_REMOVE_POWER;
            }
            SII_LOG1B(("%dn", (uint16_t)*pCbusPow));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapVideoLinkModeSet( SiiInst_t inst, Sii9777DevCapVidLinkMode_t *pVidLinkMode )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        uint8_t mode = (uint8_t)*pVidLinkMode;

        SII_LOG1A("Sii9777CbusLocalDevcapVideoLinkModeSet", pObj, ("%d", (uint16_t)*pVidLinkMode));

        retVal = sCbusLocalDevcapSet(pObj, &mode, SII9777_DEVCAP_OFFSET__VID_LINK_MODE, 1);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapVideoLinkModeGet( SiiInst_t inst, Sii9777DevCapVidLinkMode_t *pVidLinkMode )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalDevcapVideoLinkModeGet", pObj, (""));

        retVal = sCbusLocalDevcapGet(pObj, (uint8_t*)pVidLinkMode, SII9777_DEVCAP_OFFSET__VID_LINK_MODE, 1);
        if( !retVal )
        {
            SII_LOG1B(("%d\n", (uint16_t)*pVidLinkMode));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapAudioLinkModeSet( SiiInst_t inst, Sii9777DevCapAudLinkMode_t *pAudLinkMode )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        uint8_t mode = (uint8_t)*pAudLinkMode;

        SII_LOG1A("Sii9777CbusLocalDevcapAudioLinkModeSet", pObj, ("%d", (uint16_t)*pAudLinkMode));

        retVal = sCbusLocalDevcapSet(pObj, &mode, SII9777_DEVCAP_OFFSET__AUD_LINK_MODE, 1);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapAudioLinkModeGet( SiiInst_t inst, Sii9777DevCapAudLinkMode_t *pAudLinkMode )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalDevcapAudioLinkModeGet", pObj, (""));

        retVal = sCbusLocalDevcapGet(pObj, (uint8_t*)pAudLinkMode, SII9777_DEVCAP_OFFSET__AUD_LINK_MODE, 1);
        if( !retVal )
        {
            SII_LOG1B(("%d\n", (uint16_t)*pAudLinkMode));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapVideoTypeSet( SiiInst_t inst, Sii9777DevCapVideoType_t *pVideoType )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        uint8_t mode = (uint8_t)*pVideoType;

        SII_LOG1A("Sii9777CbusLocalDevcapVideoTypeSet", pObj, ("%d", (uint16_t)*pVideoType));

        retVal = sCbusLocalDevcapSet(pObj, &mode, SII9777_DEVCAP_OFFSET__VIDEO_TYPE, 1);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapVideoTypeGet( SiiInst_t inst, Sii9777DevCapVideoType_t *pVideoType )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalDevcapVideoTypeGet", pObj, (""));

        retVal = sCbusLocalDevcapGet(pObj, (uint8_t*)pVideoType, SII9777_DEVCAP_OFFSET__VIDEO_TYPE, 1);
        if( !retVal )
        {
            SII_LOG1B(("%d\n", (uint16_t)*pVideoType));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapFeatureFlagSet( SiiInst_t inst, Sii9777DevCapFeatureFlag_t *pFeatureFlag )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        uint8_t mode = (uint8_t)*pFeatureFlag;

        SII_LOG1A("Sii9777CbusLocalDevcapFeatureFlagSet", pObj, ("%d", (uint16_t)*pFeatureFlag));

        retVal = sCbusLocalDevcapSet(pObj, &mode, SII9777_DEVCAP_OFFSET__FEATURE_FLAG, 1);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalDevcapFeatureFlagGet( SiiInst_t inst, Sii9777DevCapFeatureFlag_t *pFeatureFlag )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalDevcapFeatureFlagGet", pObj, (""));

        retVal = sCbusLocalDevcapGet(pObj, (uint8_t*)pFeatureFlag, SII9777_DEVCAP_OFFSET__FEATURE_FLAG, 1);
        if( !retVal )
        {
            SII_LOG1B(("%d\n", (uint16_t)*pFeatureFlag));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusDcapChgSend( SiiInst_t inst )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusDcapChgSend", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CBUSDCAPCHG_SEND);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSDCAPCHG_SEND);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusRemoteDevcapQuery( SiiInst_t inst, uint8_t *poData, uint8_t offset, uint8_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusRemoteDevcapQuery", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)length));
        if( ((offset + length) > CBUS_DEVCAP_LEN) || (length == 0) || !poData )
        {
            SII_LOG1B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }
        if( !retVal )
        {
            sRequestMessageInit(pObj, SII9777_CBUSREMOTEDEVCAP_QUERY);
            SiiDrvIpcBytePush(pObj->mesReqInst, offset);
            SiiDrvIpcBytePush(pObj->mesReqInst, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSREMOTEDEVCAP_QUERY);
        }
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, poData, length);
            SII_LOG1B_ARRAY((uint8_t*)poData, length);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalXDevcapSet( SiiInst_t inst, uint8_t *pData, uint8_t offset, uint8_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalXDevcapSet", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)length));
        if( ((offset + length) > CBUS_DEVCAP_LEN) || (length == 0) || !pData )
        {
            SII_LOG2B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }

        if( !retVal )
        {
            SII_LOG1B_ARRAY((uint8_t*)pData, length);

            sRequestMessageInit(pObj, SII9777_CBUSLOCALXDEVCAP_SET);
            SiiDrvIpcBytePush(pObj->mesReqInst, offset);
            SiiDrvIpcBytePush(pObj->mesReqInst, length);
            SiiDrvIpcArrayPush(pObj->mesReqInst, pData, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSLOCALXDEVCAP_SET);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusLocalXDevcapGet( SiiInst_t inst, uint8_t *pData, uint8_t offset, uint8_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusLocalXDevcapGet", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)length));
        if( ((offset + length) > CBUS_DEVCAP_LEN) || (length == 0) || !pData )
        {
            SII_LOG1B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }
        if( !retVal )
        {
            sRequestMessageInit(pObj, SII9777_CBUSLOCALXDEVCAP_GET);
            SiiDrvIpcBytePush(pObj->mesReqInst, offset);
            SiiDrvIpcBytePush(pObj->mesReqInst, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSLOCALXDEVCAP_GET);
        }
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pData, length);
            SII_LOG1B_ARRAY((uint8_t*)pData, length);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusRemoteXDevcapQuery( SiiInst_t inst, uint8_t *pData, uint8_t offset, uint8_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusRemoteXDevcapQuery", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)length));
        if( ((offset + length) > CBUS_DEVCAP_LEN) || (length == 0) || !pData )
        {
            SII_LOG1B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }
        if( !retVal )
        {
            sRequestMessageInit(pObj, SII9777_CBUSREMOTEXDEVCAP_QUERY);
            SiiDrvIpcBytePush(pObj->mesReqInst, offset);
            SiiDrvIpcBytePush(pObj->mesReqInst, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSREMOTEXDEVCAP_QUERY);
        }
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pData, length);
            SII_LOG1B_ARRAY((uint8_t*)pData, length);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusBurstSend( SiiInst_t inst, uint8_t *pData, uint8_t offset, uint8_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusBurstSend", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)length));
        if( ((offset + length) > CBUS_DEVCAP_LEN) || (length == 0) || !pData )
        {
            SII_LOG2B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }

        if( !retVal )
        {
            SII_LOG1B_ARRAY((uint8_t*)pData, length);

            sRequestMessageInit(pObj, SII9777_CBUSBURST_SEND);
            SiiDrvIpcBytePush(pObj->mesReqInst, offset);
            SiiDrvIpcBytePush(pObj->mesReqInst, length);
            SiiDrvIpcArrayPush(pObj->mesReqInst, pData, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSBURST_SEND);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusBurstReceive( SiiInst_t inst, uint8_t *poData, uint8_t offset, uint8_t length )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusBurstReceive", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)length));
        if( ((offset + length) > CBUS_DEVCAP_LEN) || (length == 0) || !poData )
        {
            SII_LOG1B(("ERROR: wrong parameter\n"));
            retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
        }
        if( !retVal )
        {
            sRequestMessageInit(pObj, SII9777_CBUSBURST_RECEIVE);
            SiiDrvIpcBytePush(pObj->mesReqInst, offset);
            SiiDrvIpcBytePush(pObj->mesReqInst, length);
            SiiDrvIpcMessageSend(pObj->mesReqInst);

            retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSBURST_RECEIVE);
        }
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, poData, length);
            SII_LOG1B_ARRAY((uint8_t*)poData, length);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TmdsEnableSet( SiiInst_t inst, Sii9777TxPort_t port, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TmdsEnableSet", pObj, ("port=%d : %d", (uint16_t)port, (uint16_t)*pbOn));

        sRequestMessageInit(pObj, SII9777_TMDSENABLE_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TMDSENABLE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TmdsEnableGet( SiiInst_t inst, Sii9777TxPort_t port, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TmdsEnableGet", pObj, ("port=%d : ", (uint16_t)port));

        sRequestMessageInit(pObj, SII9777_TMDSENABLE_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TMDSENABLE_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777AvMuteSet( SiiInst_t inst, Sii9777TxPort_t port, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777AvMuteSet", pObj, ("port=%d : %d", (uint16_t)port, (uint16_t)*pbOn));

        sRequestMessageInit(pObj, SII9777_AVMUTE_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_AVMUTE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777AvMuteGet( SiiInst_t inst, Sii9777TxPort_t port, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777AvMuteGet", pObj, ("port=%d : ", (uint16_t)port));

        sRequestMessageInit(pObj, SII9777_AVMUTE_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)port);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_AVMUTE_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", *pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TmdsSwingLevelSet( SiiInst_t inst, Sii9777TxPort_t pPort, int8_t *tmdsSwingLevel )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TmdsSwingLevelSet", pObj, ("%d", (int16_t)*tmdsSwingLevel));

        sRequestMessageInit(pObj, SII9777_TMDSSWINGLEVEL_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pPort);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)*tmdsSwingLevel);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TMDSSWINGLEVEL_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TmdsSwingLevelGet( SiiInst_t inst, Sii9777TxPort_t pPort, int8_t *tmdsSwingLevel )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TmdsSwingLevelGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_TMDSSWINGLEVEL_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pPort);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TMDSSWINGLEVEL_GET);
        if( !retVal )
        {
            *tmdsSwingLevel = (int8_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*tmdsSwingLevel));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TmdsPinSwapSet( SiiInst_t inst, Sii9777TxPort_t pPort, Sii9777TmdsPinSwap_t* pTmdsPinSwap )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TmdsSwingLevelSet", pObj, ("%d", (int16_t)*pTmdsPinSwap));

        sRequestMessageInit(pObj, SII9777_TMDSPINSWAP_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pPort);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)*pTmdsPinSwap);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TMDSPINSWAP_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TmdsPinSwapGet( SiiInst_t inst, Sii9777TxPort_t pPort, Sii9777TmdsPinSwap_t* pTmdsPinSwap )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TmdsPinSwapGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_TMDSPINSWAP_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pPort);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TMDSPINSWAP_GET);
        if( !retVal )
        {
            *pTmdsPinSwap = (Sii9777TmdsPinSwap_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pTmdsPinSwap));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777PrimTxPortSet( SiiInst_t inst, Sii9777TxPort_t* pPort )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777PrimTxPortSet", pObj, ("%d", (int16_t)*pPort));

        sRequestMessageInit(pObj, SII9777_PRIMTXPORT_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, *pPort);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_PRIMTXPORT_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777PrimTxPortGet( SiiInst_t inst, Sii9777TxPort_t* pPort )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777PrimTxPortGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_PRIMTXPORT_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_PRIMTXPORT_GET);
        if( !retVal )
        {
            *pPort = (Sii9777TxPort_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pPort));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777OutputPixelFormatSet( SiiInst_t inst, Sii9777PixFmt_t* pPixFmt )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777OutputPixelFormatSet", pObj, ("\n"));
        SII_LOG1B(("clrFmt   = %d\n", pPixFmt->clrFmt));
        SII_LOG1B(("crSmpl   = %d\n", pPixFmt->crSmpl));
        SII_LOG1B(("bitDepth = %d\n", pPixFmt->bitDepth));

        sRequestMessageInit(pObj, SII9777_OUTPUTPIXELFORMAT_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pPixFmt->clrFmt);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pPixFmt->crSmpl);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pPixFmt->bitDepth);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_OUTPUTPIXELFORMAT_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777OutputPixelFormatGet( SiiInst_t inst, Sii9777PixFmt_t* pPixFmt )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777OutputPixelFormatGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_OUTPUTPIXELFORMAT_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_OUTPUTPIXELFORMAT_GET);
        if( !retVal )
        {
            pPixFmt->clrFmt   = (Sii9777ClrFmt_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            pPixFmt->crSmpl   = (Sii9777CrSmpl_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            pPixFmt->bitDepth = (Sii9777BitDepth_t)SiiDrvIpcBytePop(pObj->mesAckInst);

            SII_LOG1B(("clrFmt   = %d\n", pPixFmt->clrFmt));
            SII_LOG1B(("crSmpl   = %d\n", pPixFmt->crSmpl));
            SII_LOG1B(("bitDepth = %d\n", pPixFmt->bitDepth));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777PixelFormatConversionSet( SiiInst_t inst, bool_t* pEnable )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777PixelFormatConversionSet", pObj, ("%d", (int16_t)*pEnable));

        sRequestMessageInit(pObj, SII9777_PIXELFORMATCONVERSION_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pEnable);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_PIXELFORMATCONVERSION_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777PixelFormatConversionGet( SiiInst_t inst, bool_t* pEnable )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777PixelFormatConversionGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_PIXELFORMATCONVERSION_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_PIXELFORMATCONVERSION_GET);
        if( !retVal )
        {
            *pEnable = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pEnable));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777PixelFormatConversionQuery( SiiInst_t inst, Sii9777PixFmtConv_t* pStatus )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777PixelFormatConversionQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_PIXELFORMATCONVERSION_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_PIXELFORMATCONVERSION_QUERY);
        if( !retVal )
        {
            *pStatus = (Sii9777PixFmtConv_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pStatus));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkLutSet( SiiInst_t inst, Sii9777VidRes_t* pVidRes, uint8_t len )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkLutSet", pObj, ("length=%d, formats=", (int16_t)len));

        sRequestMessageInit(pObj, SII9777_DUALLINKLUT_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, len);
        while( len-- )
        {
            SII_LOG1B((" %d", (int16_t)*pVidRes));
            SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)(*pVidRes));
            pVidRes++;
        }
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKLUT_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkLutGet( SiiInst_t inst, Sii9777VidRes_t* pVidRes, uint8_t len )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkLutGet", pObj, ("length=%d, formats=", (int16_t)len));

        sRequestMessageInit(pObj, SII9777_DUALLINKLUT_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, len);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKLUT_GET);
        if( !retVal )
        {
            while( len-- )
            {
                *pVidRes = (Sii9777VidRes_t)SiiDrvIpcBytePop(pObj->mesAckInst);
                SII_LOG1B((" %d", (int16_t)*pVidRes));
                pVidRes++;
            }
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkManualSet( SiiInst_t inst, bool_t* bDualLink )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkManualSet", pObj, ("%d", (int16_t)*bDualLink));

        sRequestMessageInit(pObj, SII9777_DUALLINKMANUAL_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *bDualLink);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKMANUAL_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkManualGet( SiiInst_t inst, bool_t* bDualLink )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkManualGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_DUALLINKMANUAL_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKMANUAL_GET);
        if( !retVal )
        {
            *bDualLink = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*bDualLink));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkAutoEnableSet( SiiInst_t inst, bool_t* bAuto )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkAutoEnableSet", pObj, ("%d", (int16_t)*bAuto));

        sRequestMessageInit(pObj, SII9777_DUALLINKAUTOENABLE_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *bAuto);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKAUTOENABLE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkAutoEnableGet( SiiInst_t inst, bool_t* bAuto )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkAutoEnableGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_DUALLINKAUTOENABLE_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKAUTOENABLE_GET);
        if( !retVal )
        {
            *bAuto = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*bAuto));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkQuery( SiiInst_t inst, bool_t* bDualLink )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_DUALLINK_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINK_QUERY);
        if( !retVal )
        {
            *bDualLink = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*bDualLink));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkSwapEnableSet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkSwapEnableSet", pObj, ("%d", (int16_t)*pbOn));

        sRequestMessageInit(pObj, SII9777_DUALLINKSWAP_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKSWAP_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777DualLinkSwapEnableGet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777DualLinkSwapEnableGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_DUALLINKSWAP_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_DUALLINKSWAP_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777Smpte4kSet( SiiInst_t inst, Sii9777Smpte4k_t* smpte4k )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777Smpte4kSet", pObj, ("%d", (int16_t)*smpte4k));

        sRequestMessageInit(pObj, SII9777_SMPTE4K_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)*smpte4k);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_SMPTE4K_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777Smpte4kGet( SiiInst_t inst, Sii9777Smpte4k_t* smpte4k )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777Smpte4kGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_SMPTE4K_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_SMPTE4K_GET);
        if( !retVal )
        {
            *smpte4k = (Sii9777Smpte4k_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*smpte4k));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TpgEnableSet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TpgEnableSet", pObj, ("%d", (int16_t)*pbOn));

        sRequestMessageInit(pObj, SII9777_TPGENABLE_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TPGENABLE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TpgEnableGet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TpgEnableGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_TPGENABLE_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TPGENABLE_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TpgPatternSet( SiiInst_t inst, Sii9777TpgPattern_t* pPattern )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TpgPatternSet", pObj, ("%d", (int16_t)*pPattern));

        sRequestMessageInit(pObj, SII9777_TPGPATTERN_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)*pPattern);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TPGPATTERN_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TpgPatternGet( SiiInst_t inst, Sii9777TpgPattern_t* pPattern )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TpgPatternGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_TPGPATTERN_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TPGPATTERN_GET);
        if( !retVal )
        {
            *pPattern = (Sii9777TpgPattern_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pPattern));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TpgVidResSet( SiiInst_t inst, Sii9777VidRes_t* pVidRes )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TpgVidResSet", pObj, ("%d", (int16_t)*pVidRes));

        sRequestMessageInit(pObj, SII9777_TPGVIDRES_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)*pVidRes);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TPGVIDRES_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777TpgVidResGet( SiiInst_t inst, Sii9777VidRes_t* pVidRes )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777TpgVidResGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_TPGVIDRES_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_TPGVIDRES_GET);
        if( !retVal )
        {
            *pVidRes = (Sii9777VidRes_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pVidRes));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsBypassSet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsBypassSet", pObj, ("%d", (int16_t)*pbOn));

        sRequestMessageInit(pObj, SII9777_CMSBYPASS_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pbOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSBYPASS_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsBypassGet( SiiInst_t inst, bool_t* pbOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsBypassGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CMSBYPASS_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSBYPASS_GET);
        if( !retVal )
        {
            *pbOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pbOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsSaturationSet( SiiInst_t inst, Sii9777CmsColor_t clr, int8_t* pSat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsSaturationSet", pObj, ("%d", (int16_t)*pSat));

        sRequestMessageInit(pObj, SII9777_CMSSATURATION_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)clr);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)*pSat);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSSATURATION_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsSaturationGet( SiiInst_t inst, Sii9777CmsColor_t clr, int8_t* pSat )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsSaturationGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CMSSATURATION_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)clr);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSSATURATION_GET);
        if( !retVal )
        {
            *pSat = (int8_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pSat));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsHueSet( SiiInst_t inst, Sii9777CmsColor_t clr, int8_t* pHue )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsHueSet", pObj, ("%d", (int16_t)*pHue));

        sRequestMessageInit(pObj, SII9777_CMSHUE_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)clr);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)*pHue);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSHUE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsHueGet( SiiInst_t inst, Sii9777CmsColor_t clr, int8_t* pHue )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsHueGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CMSHUE_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)clr);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSHUE_GET);
        if( !retVal )
        {
            *pHue = (int8_t)SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pHue));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsWhitePointTempSet( SiiInst_t inst, Sii9777CmsColor_t clr, int16_t* pTemperature )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsWhitePointTempSet", pObj, ("%d", (int16_t)*pTemperature));

        sRequestMessageInit(pObj, SII9777_CMSWHITEPOINTTEMP_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)clr);
        SiiDrvIpcWordPush(pObj->mesReqInst, (uint8_t)*pTemperature);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSWHITEPOINTTEMP_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CmsWhitePointTempGet( SiiInst_t inst, Sii9777CmsColor_t clr, int16_t* pTemperature )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CmsWhitePointTempGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CMSWHITEPOINTTEMP_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)clr);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CMSWHITEPOINTTEMP_GET);
        if( !retVal )
        {
            *pTemperature = (int16_t)SiiDrvIpcWordPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (int16_t)*pTemperature));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ArcEnableSet( SiiInst_t inst, bool_t* pOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ArcEnableSet", pObj, ("%d", (uint16_t)*pOn));

        sRequestMessageInit(pObj, SII9777_ARCENABLE_SET);
        SiiDrvIpcBoolPush(pObj->mesReqInst, *pOn);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_ARCENABLE_SET);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ArcEnableGet( SiiInst_t inst, bool_t* pOn )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ArcEnableGet", pObj, (""));

        sRequestMessageInit(pObj, SII9777_ARCENABLE_GET);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_ARCENABLE_GET);
        if( !retVal )
        {
            *pOn = SiiDrvIpcBoolPop(pObj->mesAckInst);
            SII_LOG1B(("%d", (uint16_t)*pOn));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ChipRegWrite( SiiInst_t inst, uint16_t addr, uint8_t* pData, uint8_t len)
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ChipRegWrite", pObj, ("addr=%04X, len=%d, data : ", addr, (uint16_t)len));
        SII_LOG1B_ARRAY(pData, len);

        sRequestMessageInit(pObj, SII9777_CHIPREG_WRITE);
        SiiDrvIpcWordPush(pObj->mesReqInst, addr);
        SiiDrvIpcBytePush(pObj->mesReqInst, len);
        SiiDrvIpcArrayPush(pObj->mesReqInst, pData, len);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CHIPREG_WRITE);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777ChipRegRead( SiiInst_t inst, uint16_t addr, uint8_t* pData, uint8_t len )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777ChipRegRead", pObj, ("addr=%04X, len=%d, data : ", addr, (uint16_t)len));

        sRequestMessageInit(pObj, SII9777_CHIPREG_READ);
        SiiDrvIpcWordPush(pObj->mesReqInst, addr);
        SiiDrvIpcBytePush(pObj->mesReqInst, len);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CHIPREG_READ);
        if( !retVal )
        {
            SiiDrvIpcArrayPop(pObj->mesAckInst, pData, len);
            SII_LOG1B_ARRAY(pData, len);
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusEventQuery( SiiInst_t inst, Sii9777CbusEvt_t *poState )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusEventQuery", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CBUSEVENT_QUERY);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSEVENT_QUERY);
        if( !retVal )
        {
            *poState = (Sii9777CbusEvt_t)SiiDrvIpcWordPop(pObj->mesAckInst);
            SII_LOG1B(("%04X", (uint16_t)*poState));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusMscMsgReceive( SiiInst_t inst, Sii9777MscCmd_t* pMscCmd )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusMscMsgReceive", pObj, (""));

        sRequestMessageInit(pObj, SII9777_CBUSMSCMSG_RECEIVE);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSMSCMSG_RECEIVE);
        if( !retVal )
        {
            pMscCmd->subCmd    = SiiDrvIpcBytePop(pObj->mesAckInst);
            pMscCmd->codeValue = SiiDrvIpcBytePop(pObj->mesAckInst);
            SII_LOG1B(("Cmd=0x%02X, Code=0x%02X\n", (uint16_t)pMscCmd->subCmd, (uint16_t)pMscCmd->codeValue));
        }
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

SiiRetVal_t Sii9777CbusMscMsgSend( SiiInst_t inst, Sii9777MscCmd_t* pMscCmd )
{
    Obj_t*      pObj   = (Obj_t*)SII_SYS_OBJ_CHECK(sObjListInst, inst);
    SiiRetVal_t retVal = 0;

    retVal = MUTEX_API_TAKE(pObj);
    if( !retVal )
    {
        SII_LOG1A("Sii9777CbusMscMsgSend", pObj, (""));
        SII_LOG1B(("Cmd=0x%02X, Code=0x%02X\n", (uint16_t)pMscCmd->subCmd, (uint16_t)pMscCmd->codeValue));

        sRequestMessageInit(pObj, SII9777_CBUSMSCMSG_SEND);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pMscCmd->subCmd);
        SiiDrvIpcBytePush(pObj->mesReqInst, (uint8_t)pMscCmd->codeValue);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSMSCMSG_SEND);
    }
    MUTEX_API_GIVE(pObj);
    RETURN_API(retVal);
}

/***** local functions *******************************************************/

static void sResourcesDelete( Obj_t* pObj )
{
#if( SII9777_USER_DEF__MULTI_THREAD )
    if( pObj->semaphoreHandle )
    {
        SiiPlatformSemaphoreDelete(pObj->semaphoreHandle);
    }

    if( pObj->semaphoreIpc )
    {
        SiiPlatformSemaphoreDelete(pObj->semaphoreIpc);
    }

    if( pObj->semaphoreApi )
    {
        SiiPlatformSemaphoreDelete(pObj->semaphoreApi);
    }
#endif

    if( pObj->mesAckInst )
    {
        SiiDrvIpcMessageDelete(pObj->mesAckInst);
    }

    if( pObj->mesReqInst )
    {
        SiiDrvIpcMessageDelete(pObj->mesReqInst);
    }

    SiiSysObjInstanceDelete(pObj);
    if( !SiiSysObjFirstGet(sObjListInst) )
    {
        /* Delete list of objects */
        SiiSysObjListDelete(sObjListInst);
        sObjListInst = SII_INST_NULL;
    }
}

static void sHandleFlashProg( Obj_t* pObj )
{
    /* Make sure previous flashing operation is done */
    SII_ASSERT(SiiDrvIspOperationDone());

    if( pObj->pFlashPacket )
    {
        uint16_t pageSize = 256;

#if( SII9777_FLASH_BUFFER_SIZE )
        /* Detemine maximum page size based on absolute flashing address                 */
        /* Maximum page size is 256 bytes if LSByte of absolute flashing address is zero */
        pageSize = 256 - (pObj->flashAddr%256); 

        /* Check if packet size is smaller than maximum page size */
        if( pObj->packetSize < pageSize )
        {
            pageSize = pObj->packetSize;
        }
#endif

        SII_LOG2A("sHandleFlashProg", pObj, ("Offset=0x%lX, length=0x%x\n", pObj->flashAddr, pageSize));
#if( SII9777_FLASH_BUFFER_SIZE )
        /* Program page of bytes */
        SiiDrvIspBurstWrite(pObj->flashAddr, pObj->pPacketPntr, pageSize);

        pObj->pPacketPntr   += pageSize;
        pObj->packetSize    -= pageSize;

        /* Check if entire packet has been programmed */
        if( 0 == pObj->packetSize )
        {
            /* Delete flash packet */ 
            SiiSysMallocDelete(pObj->pFlashPacket);
            pObj->pFlashPacket = NULL;

            /* Assert notification to indicate that packet has been flashed */ 
            pObj->eventFlagsUser |= SII9777_EVENT_FLAGS__FLASH_DONE;
        }
#else
        /* Program page of bytes */
        SiiDrvIspBurstWrite(pObj->flashAddr, pObj->pFlashPacket, pageSize);

        /* Clear packet pointer to indicate packet is programmed */
        pObj->pFlashPacket = NULL;

        /* Assert notification to indicate that packet has been flashed */ 
        pObj->eventFlagsUser |= SII9777_EVENT_FLAGS__FLASH_DONE;
#endif

        pObj->flashAddr     += pageSize;
    }
    else
    {
        /* Assert notification to indicate erase cyle has been finished */ 
        pObj->eventFlagsUser |= SII9777_EVENT_FLAGS__FLASH_DONE;
    }
}

static void sHandleIpcInterrupts( Obj_t* pObj )
{
    uint8_t ipcFlags = 0;

    /* Read IPC interrupt status flags */
    ipcFlags = SiiDrvCraRead8(pObj->devId, 0x7062);
    if( ipcFlags )
    {
        /* Clear IPC interrupt status flags */
        SiiDrvCraWrite8(pObj->devId, 0x7062, ipcFlags);

        if( ipcFlags & 0x7 )
        {
            /* Handle IPC interrupts */
            SiiDrvIpcHandler((SiiDrvIpcIsrFlags_t)ipcFlags);
        }
    }

    SII_LOG2A("sHandleIpcInterrupts", pObj, ("%02X\n", (uint16_t)ipcFlags));
}

static void sHandleIspInterrupts( Obj_t* pObj )
{
    uint8_t ispFlags = 0;

    /* Read ISP interrupt status flags */
    ispFlags = SiiDrvCraRead8(pObj->devId, 0x00f3);
    if( ispFlags )
    {
        /* Clear ISP interrupt status flag */
        SiiDrvCraWrite8(pObj->devId, 0x00f3, ispFlags);

        /* Handle ISP interrupts */
        sHandleFlashProg(pObj);  /* Flash program handler */
    }

    SII_LOG2A("sHandleIspInterrupts", pObj, ("%02X\n", (uint16_t)ispFlags));
}

static void sRequestMessageInit( Obj_t* pObj, SiiDrvIpcOpCode_t opCode )
{
    SiiDrvIpcMessageClear(pObj->mesReqInst);
    SiiDrvIpcTypeSet(pObj->mesReqInst, SII_DRV_IPC_TYPE__REQUEST);
    SiiDrvIpcOpCodeSet(pObj->mesReqInst, opCode);
    pObj->tagCode++;
    SiiDrvIpcTagCodeSet(pObj->mesReqInst, pObj->tagCode);

    /* Clear previous acknowledge message before new request message is initiated */
    SiiDrvIpcMessageClear(pObj->mesAckInst);
}

static SiiRetVal_t sWaitForValidIpcResponseMessage( Obj_t* pObj, SiiDrvIpcOpCode_t opCode )
{
    SiiRetVal_t retVal = 0;

    SII_LOG2A("sWaitForValidIpcResponseMessage", pObj, (""));
#if( SII9777_USER_DEF__MULTI_THREAD )
    /* First check if API call was initiated through user call back. */
    /* Response message polling is required when called through user call-back, since Sii9777Handle is still blocked */
    if( pObj->bFromUserCallBack )
#endif
    {   /* Poll and wait until response message is received */
        
        SiiSysTimeMilli_t timeOut;

        /* Wait for respondse message from slave */
        SiiSysTimeOutMilliSet(&timeOut, IPC_COMPLETION_TIMEOUT);
        while( opCode != SiiDrvIpcOpCodeGet(pObj->mesAckInst) ) /* Check if responding message has been received. */
        {
            SiiPlatformSleepMsec(10);
            sHandleIpcInterrupts(pObj);

            if( SiiSysTimeOutMilliIs(&timeOut) ) /* Check for time-out */
            {
                retVal = SII9777_RETVAL__IPC_NO_RESPONSE;
                break;
            }
        }
    }
#if( SII9777_USER_DEF__MULTI_THREAD )
    else
    {
        /* Wait for Sii9777Handle() to receive response message */
        /* sSemaphoreIpcTake returns false if released through timeout */
        retVal = sSemaphoreIpcTake(pObj);
        if( !retVal )
        {
            if( opCode != SiiDrvIpcOpCodeGet(pObj->mesAckInst) )
            {
                retVal = SII9777_RETVAL__IPC_NO_RESPONSE;
            }
        }
    }
#endif

    if( pObj->eventFlagsUser )
    {
        /* bit 3 is used to invoke user application to call Sii9777Handle() to ensure handling of remaining event flags */
        SiiDrvCraWrite8(pObj->devId, 0x7060, 0x08);
    }

    if( retVal )
    {
        /* timeout happened, clear-up IPC itself */
        SiiDrvIpcReset();
    }
    else
    {
        /* Check Tag Code */
        if( pObj->tagCode != SiiDrvIpcTagCodeGet(pObj->mesAckInst) )
        {
            retVal = SII9777_RETVAL__IPC_WRONG_TAG_CODE;
        }
    }

    if( !retVal )
    {
        /* Read result code */
        retVal = (SiiRetVal_t)SiiDrvIpcBytePop(pObj->mesAckInst);
    }

    return retVal;
}

#if( SII9777_USER_DEF__MULTI_THREAD )
static SiiRetVal_t sSemaphoreIpcTake( Obj_t* pObj )
{
    SiiRetVal_t retVal = 0;

    SII_ASSERT(pObj->semaphoreHandle);
    SII_ASSERT(pObj->semaphoreIpc);

    SII_LOG2A("sSemaphoreIpcTake", pObj, (""));

    /* Ipc semaphore should not be given at this point. If still given then take ipc semaphore to clear */
    if( pObj->bIpcGiven )
    {
        /* Take IPC Semaphore */
        if( SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreTake(pObj->semaphoreIpc, 1) )
        {
            SII_ASSERT(0);
        }

        pObj->bIpcGiven = false;
    }

    /* Give Handle Semaphore */
    SiiPlatformSemaphoreGive(pObj->semaphoreHandle);

    if( !retVal )
    {
        if( SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreTake(pObj->semaphoreIpc, IPC_COMPLETION_TIMEOUT) )
        {
            retVal = SII9777_RETVAL__SEMAPHORE_IPC_EXPIRE;
        }
        else
        {
            /* Clear IPC given flag because IPC take did not get expired */
            pObj->bIpcGiven = false;
        }
    }
    if( !retVal && (SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreTake(pObj->semaphoreHandle, IPC_COMPLETION_TIMEOUT)) )
    {
        retVal = SII9777_RETVAL__SEMAPHORE_HANDLE_EXPIRE;
    }
    return retVal;
}

static void sSemaphoreIpcGive( Obj_t* pObj )
{
    SII_ASSERT(pObj->semaphoreIpc);

    SII_LOG2A("sSemaphoreIpcGive", pObj, (""));

    /* Set Ipc given flag so that later we can still determine if IPC semaphore was still given after it was released through expiration */
    pObj->bIpcGiven = true;

    SiiPlatformSemaphoreGive(pObj->semaphoreIpc);
}

static SiiRetVal_t sMutexHandleTake( Obj_t* pObj )
{
    SiiRetVal_t retVal = 0;

    SII_ASSERT(pObj->semaphoreHandle);

    SII_LOG2A("sMutexHandleTake", pObj, (""));

    if( !retVal && (SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreTake(pObj->semaphoreHandle, IPC_COMPLETION_TIMEOUT)) )
    {
        retVal = SII9777_RETVAL__SEMAPHORE_HANDLE_EXPIRE;
    }
    return retVal;
}

static void sMutexHandleGive( Obj_t* pObj )
{
    SII_ASSERT(pObj->semaphoreHandle);

    SII_LOG2A("sMutexHandleGive", pObj, (""));

    SiiPlatformSemaphoreGive(pObj->semaphoreHandle);
}

static SiiRetVal_t sMutexApiTake( Obj_t* pObj )
{
    SiiRetVal_t retVal = 0;

    SII_ASSERT(pObj->semaphoreApi);
    SII_ASSERT(pObj->semaphoreHandle);

    SII_LOG2A("sMutexApiTake", pObj, (""));

    if( !retVal && (SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreTake(pObj->semaphoreApi, IPC_COMPLETION_TIMEOUT)) )
    {
        retVal = SII9777_RETVAL__SEMAPHORE_API_EXPIRE;
    }
    if( !retVal && (SII_PLATFORM_STATUS__SUCCESS != SiiPlatformSemaphoreTake(pObj->semaphoreHandle, IPC_COMPLETION_TIMEOUT)) )
    {
        retVal = SII9777_RETVAL__SEMAPHORE_HANDLE_EXPIRE;
    }
    return retVal;
}

static void sMutexApiGive( Obj_t* pObj )
{
    SII_ASSERT(pObj->semaphoreApi);
    SII_ASSERT(pObj->semaphoreHandle);

    SII_LOG2A("sMutexApiGive", pObj, (""));

    SiiPlatformSemaphoreGive(pObj->semaphoreHandle);
    SiiPlatformSemaphoreGive(pObj->semaphoreApi);
}
#endif

static void sDeviceRestart( Obj_t* pObj )
{
    /* Re-fetch firmware from flash and restart firmware */
    pObj->bootStat = SII9777_BOOT_STAT__IN_PROGRESS;

    /* SW reset SiI9777 device */
    SiiPlatformDeviceResetSet(pObj->devId, true);
    SiiPlatformDeviceResetSet(pObj->devId, false);

    /* Set boot timeout timer */
    SiiSysTimeOutMilliSet(&pObj->bootTimeOut, BOOT_TIME_MAX);
}

static bool_t sDevicePing( Obj_t* pObj )
{
    SiiRetVal_t retVal = 0;

    /* Ping device */
    sRequestMessageInit(pObj, SII9777_FIRMWAREVERSION_QUERY);
    SiiDrvIpcMessageSend(pObj->mesReqInst);

    /* wait for response from 8051 firmware and check result */
    retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_FIRMWAREVERSION_QUERY);

    return (retVal) ? (false) : (true);
}

static void sInterruptsEnableSet( Obj_t* pObj, bool_t bEnable )
{
    if( bEnable )
    {
        /* Set interrupt mask bits to allow invokation of INT signal */
        SiiDrvCraWrite8(pObj->devId, 0x7061, 0x0F);
        SiiDrvCraWrite8(pObj->devId, 0x00f2, 0x01);
    }
    else
    {
        /* Clear interrupt mask bits to avoid invokation of INT signal */
        SiiDrvCraWrite8(pObj->devId, 0x7061, 0x00);
        SiiDrvCraWrite8(pObj->devId, 0x00f2, 0x00);
    }
}

static SiiRetVal_t sCbusLocalDevcapSet( Obj_t* pObj, uint8_t *pData, uint8_t offset, uint8_t len )
{
    SiiRetVal_t retVal = 0;

    SII_LOG2A("Sii9777CbusLocalDevcapSet", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)len));
    if( ((offset + len) > CBUS_DEVCAP_LEN) || (len == 0) || !pData )
    {
        SII_LOG2B(("ERROR: wrong parameter\n"));
        retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
    }

    if( !retVal )
    {
        SII_LOG2B_ARRAY((uint8_t*)pData, len);

        sRequestMessageInit(pObj, SII9777_CBUSLOCALDEVCAP_SET);
        SiiDrvIpcBytePush(pObj->mesReqInst, offset);
        SiiDrvIpcBytePush(pObj->mesReqInst, len);
        SiiDrvIpcArrayPush(pObj->mesReqInst, pData, len);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSLOCALDEVCAP_SET);
    }
    return retVal;
}

static SiiRetVal_t sCbusLocalDevcapGet( Obj_t* pObj, uint8_t *poData, uint8_t offset, uint8_t len )
{
    SiiRetVal_t retVal = 0;

    SII_LOG2A("Sii9777CbusLocalDevcapGet", pObj, ("offset=%d, length=%d : ", (uint16_t)offset, (uint16_t)len));
    if( ((offset + len) > CBUS_DEVCAP_LEN) || (len == 0) || !poData )
    {
        SII_LOG2B(("ERROR: wrong parameter\n"));
        retVal = SII9777_RETVAL_ERR__PARAMETER; // error happen
    }
    if( !retVal )
    {
        sRequestMessageInit(pObj, SII9777_CBUSLOCALDEVCAP_GET);
        SiiDrvIpcBytePush(pObj->mesReqInst, offset);
        SiiDrvIpcBytePush(pObj->mesReqInst, len);
        SiiDrvIpcMessageSend(pObj->mesReqInst);

        retVal = sWaitForValidIpcResponseMessage(pObj, SII9777_CBUSLOCALDEVCAP_GET);
    }
    if( !retVal )
    {
        SiiDrvIpcArrayPop(pObj->mesAckInst, poData, len);
        SII_LOG2B_ARRAY((uint8_t*)poData, len);
    }
    return retVal;
}

/***** end of file ***********************************************************/
